/*
 * Augmented Reality For Maemo
 * Based on the code available at https://code.launchpad.net/~rojtberg/arinterface/ by pavel
 *
 * Author: Tassio Knop
 * www.impa.br/~tknop
 */


#include "ObjReader.h"

#include <QFile>
#include <QTextStream>
#include <iostream>
#include <QTime>
#include <QFileInfo>
#include <cmath>
#include <QDebug>

void BBox::include(QVector<float> v) {
	minPoint.setX(min(minPoint.x(), qreal(v[0])));
	minPoint.setY(min(minPoint.y(), qreal(v[1])));
	minPoint.setZ(min(minPoint.z(), qreal(v[2])));

	maxPoint.setX(max(maxPoint.x(), qreal(v[0])));
	maxPoint.setY(max(maxPoint.y(), qreal(v[1])));
	maxPoint.setZ(max(maxPoint.z(), qreal(v[2])));
}

QVector3D BBox::size() {
	return maxPoint - minPoint;
}

ObjReader::ObjReader(const string path) {
	nfaces = 0;
	hasNormal = false;
	QFile objFile(path.c_str());
	QTextStream in(&objFile);
	QString line;
	QStringList parts;
	TexSlice ts;
	QTime t;

	relPath = QFileInfo(path.c_str()).path();
	objFile.open(QIODevice::ReadOnly | QIODevice::Text);
	t.start();

	while (not in.atEnd()) {
		line = in.readLine();
		parts = line.split(" ", QString::SkipEmptyParts);

		if (parts.length() == 0)
			continue; // skip empty lines

//		qDebug() << parts[0];

		if (parts[0] == "v") {
			readV(parts);
		} else if (parts[0] == "vn") {
			readVN(parts);
		} else if (parts[0] == "vt") {
			readVT(parts);
		} else if (parts[0] == "f") {
			++nfaces;
			readF(parts);
//		} else if (parts[0] == "mtllib") {
//			readMtlLib(parts);
//		} else if (parts[0] == "usemtl") {
//			curMtl = parts[1];
		}
	}

	objFile.close();

	cout << path << "stats :" << endl;
	cout << " File read in: " << t.elapsed() << "ms" << endl;
	cout << " Vertices: " << verts.size() << endl;
	cout << " Normals: " << normals.size() << endl;
	cout << " Tex UV: " << tex_uv.size() << endl;

}

QVector<float>& ObjReader::getVBO() {

	return vbo;
}

vector<TexSlice> ObjReader::getSlices() {
//	TexSlice ts;
//	QList<QString> keys = mtrls.keys();
//	qDebug() << keys;
//	unsigned int offset = 0;

//	QTextStream cout(stdout);

//	for (int i = 0; i < keys.size(); i++) {
//		ts.start = offset;
//		ts.count = slices[keys[i]].size() / 8;
//		ts.path = mtrls[keys[i]];

//		texSlices.push_back(ts);

//		offset += ts.count;
//	}

//	return texSlices;
}

void ObjReader::readV(QStringList& parts) {
	QVector<float> v(3);

	for (int i = 0; i < 3; i++) {
		v[i] = parts[i + 1].toFloat();
	}

	verts.append(v);
	boundings.include(v);
}

void ObjReader::readVN(QStringList& parts) {
	QVector<float> vn(3);

	for (int i = 0; i < 3; i++) {
		vn[i] = parts[i + 1].toFloat();
	}

	normals.append(vn);
}

void ObjReader::readVT(QStringList& parts) {
	QVector<float> vt(2);

	for (int i = 0; i < 2; i++) {
		vt[i] = parts[i + 1].toFloat();
	}

	tex_uv.append(vt);
}

void ObjReader::readF(QStringList& parts) {
	QStringList fp;

	fp = parts[ 1].split("//");
//	qDebug() << fp.size();
	QVector3D s = boundings.size();
	float mins = min(s.x(),s.y());
	mins = min(mins,s.z());
	if (fp.size() == 1) { //first case: only vertices and faces. and only triangular faces
		vbo << verts[parts[1].toInt()-1].at(0)*1/mins << verts[parts[1].toInt()-1].at(1)*1/mins << verts[parts[1].toInt()-1].at(2)*1/mins
				<< verts[parts[2].toInt()-1].at(0)*1/mins << verts[parts[2].toInt()-1].at(1)*1/mins << verts[parts[2].toInt()-1].at(2)*1/mins
				<< verts[parts[3].toInt()-1].at(0)*1/mins << verts[parts[3].toInt()-1].at(1)*1/mins << verts[parts[3].toInt()-1].at(2)*1/mins;
	}
	else
	{
		hasNormal = true;
		int nvert = fp.at(0).toInt();
		int nnorm = fp.at(1).toInt();
		vbo << verts[nvert-1].at(0)*1/mins  << verts[nvert-1].at(1)*1/mins << verts[nvert-1].at(2)*1/mins
				<< normals[nnorm-1].at(0) << normals[nnorm-1].at(1) << normals[nnorm-1].at(2);

		fp = parts[2].split("//");
		nvert = fp.at(0).toInt();
		nnorm = fp.at(1).toInt();
		vbo << verts[nvert-1].at(0)*1/mins  << verts[nvert-1].at(1)*1/mins << verts[nvert-1].at(2)*1/mins
				<< normals[nnorm-1].at(0) << normals[nnorm-1].at(1) << normals[nnorm-1].at(2);

		fp = parts[3].split("//");
		nvert = fp.at(0).toInt();
		nnorm = fp.at(1).toInt();
		vbo << verts[nvert-1].at(0)*1/mins  << verts[nvert-1].at(1)*1/mins << verts[nvert-1].at(2)*1/mins
				<< normals[nnorm-1].at(0) << normals[nnorm-1].at(1) << normals[nnorm-1].at(2);
	}
}

void ObjReader::readMtlLib(QStringList& parts) {
//	MtlReader mr(relPath + "/" + parts[1]);
//	mtrls = mr.getMaterials();

//	cout << "entrou" << endl;
//	QList<QString> keys = mtrls.keys();

//	for (int i = 0; i < keys.size(); i++) {
//		slices.insert(keys[i], QVector<float> ());
//	}
}

MtlReader::MtlReader(const QString path) {
	QFile mtlFile(path);
	QTextStream in(&mtlFile);
	QString line;
	QStringList parts;
	QString curmtl;

	relPath = QFileInfo(path).path().toStdString();

	mtlFile.open(QIODevice::ReadOnly | QIODevice::Text);

	while (not in.atEnd()) {
		line = in.readLine();
		parts = line.split(" ", QString::SkipEmptyParts);

		if (parts.length() == 0)
			continue; // skip empty lines

		if (parts[0] == "newmtl") {
			curmtl = parts[1];
		} else if (parts[0] == "map_Kd") {
			materials.insert(curmtl, relPath + "/" + parts[1].toStdString());
		}
	}

	mtlFile.close();
}

QHash<QString, string> MtlReader::getMaterials() {
	return materials;
}
