/*
 * Augmented Reality For Maemo
 * Based on the code available at https://code.launchpad.net/~rojtberg/arinterface/ by pavel
 *
 * Author: Tassio Knop
 * www.impa.br/~tknop
 */


#pragma once

#include <QVector>
#include <QVector3D>
#include <QHash>
#include <QStringList>
#include <string>
#include <vector>

using namespace std;

/**
 * Bounding Box
 */
struct BBox {
	QVector3D minPoint;
	QVector3D maxPoint;
	QVector3D size();
	void include(QVector<float> v);
};

/**
 * Holds a slice of vertices from a Vertex Buffer Object
 * that share the same material. Greatly reduces GL function calls
 */
struct TexSlice {
	unsigned int start;
	unsigned int count;
	unsigned int texId; // texture ID
	string path;       // path to the texture file
};

/**
 * Reads an Model from an Obj file. *
 *
 * - faces must be triangles
 */
class ObjReader {
public:
	ObjReader(const string path);
	QVector<float>& getVBO();
	vector<TexSlice> getSlices();
	BBox boundings;
	bool hasNormal;
public:
	QString relPath;
	QString curMtl;
	QVector<QVector<float> > verts;
	QVector<QVector<float> > normals;
	QVector<QVector<float> > tex_uv;
	QVector<float> vbo;
	int nfaces;
	vector<TexSlice> texSlices;
	QVector<QVector<float> > vals;
	QHash<QString, string> mtrls;
	void readV(QStringList& parts);
	void readVN(QStringList& parts);
	void readVT(QStringList& parts);
	void readF(QStringList& parts);
	void readMtlLib(QStringList& parts);
};

/**
 * Reads Material definitions from a MTL file
 * - currently only reads Texture definitions
 * - ignores everything else
 */
class MtlReader {
public:
	MtlReader(const QString path);
	QHash<QString, string> getMaterials();
private:
	string relPath;
	QHash<QString, string> materials;
};
