/*
 * Augmented Reality For Maemo
 * Based on the code available at https://code.launchpad.net/~rojtberg/arinterface/ by pavel
 *
 * Author: Tassio Knop
 * www.impa.br/~tknop
 */


#ifdef NO_ES
#include <GL/glew.h>
#include <GL/gl.h>
#else
#include <GLES2/gl2.h>
#endif
#include <QImage>
#include <QVector>
#include <iostream>
#include <QPainter>

using namespace std;

#include "Object.h"

void Object::setupText(float size, float x, float y , float z)
{
	type = TEXT;
	QVector<float> vboData;

	QFont font("Arial", size);
	QPainterPath path;
	path.addText(QPointF(0, 0), font, QString(QObject::tr("text test")));
	QList<QPolygonF> poly = path.toSubpathPolygons();
	for (QList<QPolygonF>::iterator i = poly.begin(); i != poly.end(); i++){
		for (QPolygonF::iterator p = (*i).begin(); p != i->end(); p++)
		{    vboData << p->rx()*0.1+x
								 << -p->ry()*0.1+y
								 << z;
		}
		letters<<(*i).size();
	}


	glGenBuffers(1, &vbo);
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, vboData.size() * sizeof(float), vboData.data(), GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind

	nvert = (int)(vboData.size()/3.0f);

}

void Object::setupCube(float x, float y , float z) {
	type = CUBE;
	QImage tex;
	// load texture
	tex.load("/opt/arinterface/visgraf.png");
	tex = tex.scaledToWidth(tex.width());
	tex = QGLWidget::convertToGLFormat(tex);

	glGenTextures(1, &cubeTexId);
	glBindTexture(GL_TEXTURE_2D, cubeTexId);

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tex.width(), tex.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits());
	glGenerateMipmap(GL_TEXTURE_2D);


	QVector<float> vboData;// = objFile.getVBO();

	//vertices
	vboData << -0.5+x << 0.5+y<< 0.5+z<<
							0.5+x<<-0.5+y<<0.5+z<<
							-0.5+x<<-0.5+y<<0.5+z<<
							0.5+x<< -0.5+y<< 0.5+z<<
							-0.5+x<<0.5+y<<0.5+z<<
							0.5+x<<0.5+y<<0.5+z<<
							-0.5+x<< -0.5+y<< -0.5+z<<
							0.5+x<<-0.5+y<<-0.5+z<<
							-0.5+x<<0.5+y<<-0.5+z<<
							0.5+x<< 0.5+y<< -0.5+z<<
							-0.5+x<<0.5+y<<-0.5+z<<
							0.5+x<<-0.5+y<<-0.5+z<<
							0.5+x<< -0.5+y<< -0.5+z<<
							0.5+x<<-0.5+y<<0.5+z<<
							0.5+x<<0.5+y<<-0.5+z<<
							0.5+x<< 0.5+y<< 0.5+z<<
							0.5+x<<0.5+y<<-0.5+z<<
							0.5+x<<-0.5+y<<0.5+z<<
							-0.5+x<< 0.5+y<< -0.5+z<<
							-0.5+x<<-0.5+y<<0.5+z<<
							-0.5+x<<-0.5+y<<-0.5+z<<
							-0.5+x<< -0.5+y<< 0.5+z<<
							-0.5+x<<0.5+y<<-0.5+z<<
							-0.5+x<<0.5+y<<0.5+z<<
							0.5+x<< 0.5+y<< -0.5+z<<
							-0.5+x<< 0.5+y<<  0.5+z<<
							-0.5+x<<  0.5+y<<  -0.5+z<<
							-0.5+x<<  0.5+y<<  0.5+z<<
							0.5+x<<  0.5+y<<  -0.5+z<<
							0.5+x<< 0.5+y<<  0.5+z<<
							-0.5+x<<  -0.5+y<< -0.5+z<<
							-0.5+x<< -0.5+y<< 0.5+z<<
							0.5+x<< -0.5+y<< -0.5+z<<
							0.5+x<< -0.5+y<< 0.5+z<<
							0.5+x<<  -0.5+y<< -0.5+z<<
							-0.5+x<<  -0.5+y<< 0.5+z ;


	//texture coordinates
	vboData <<	0.0f<<0.0f<< 1.0f<<1.0f<< 1.0f<<0.0f<<
							1.0f<<1.0f<< 0.0f<<0.0f<< 0.0f<<1.0f<<
							1.0f<<1.0f<< 1.0f<<0.0f<< 0.0f<<1.0f<<
							0.0f<<0.0f<< 0.0f<<1.0f<< 1.0f<<0.0f<<

							1.0f<<1.0f<< 1.0f<<0.0f<< 0.0f<<1.0f<<
							0.0f<<0.0f<< 0.0f<<1.0f<< 1.0f<<0.0f<<
							0.0f<<0.0f<< 1.0f<<1.0f<< 1.0f<<0.0f<<
							1.0f<<1.0f<< 0.0f<<0.0f<< 0.0f<<1.0f<<

							0.0f<<1.0f<< 1.0f<<0.0f<< 1.0f<<1.0f<<
							1.0f<<0.0f<< 0.0f<<1.0f<< 0.0f<<0.0f<<
							1.0f<<0.0f<< 1.0f<<1.0f<< 0.0f<<0.0f<<
							0.0f<<1.0f<< 0.0f<<0.0f<< 1.0f<<1.0f;


	//normals
	vboData <<	0<<0<<-1<< 0<<0<<-1<< 0<<0<<-1<<
							0<<0<<-1<< 0<<0<<-1<< 0<<0<<-1<<
							0<<0<<1<< 0<<0<<1<< 0<<0<<1<<
							0<<0<<1<< 0<<0<<1<< 0<<0<<1<<

							-1<<0<<0<< -1<<0<<0<< -1<<0<<0<<
							-1<<0<<0<< -1<<0<<0<< -1<<0<<0<<
							1<<0<<0<< 1<<0<<0<< 1<<0<<0<<
							1<<0<<0<< 1<<0<<0<< 1<<0<<0<<

							0<<-1<<0<< 0<<-1<<0<< 0<<-1<<0<<
							0<<-1<<0<< 0<<-1<<0<< 0<<-1<<0<<
							0<<1<<0<< 0<<1<<0<< 0<<1<<0<<
							0<<1<<0<< 0<<1<<0<< 0<<1<<0;


	//vertices 2 (at 0,0,0)
	float x2 = 1;
	vboData << -0.5+x2 << 0.5<< 0.5<<
							0.5<<-0.5<<0.5<<
							-0.5<<-0.5<<0.5<<
							0.5<< -0.5<< 0.5<<
							-0.5<<0.5<<0.5<<
							0.5<<0.5<<0.5<<
							-0.5<< -0.5<< -0.5<<
							0.5<<-0.5<<-0.5<<
							-0.5<<0.5<<-0.5<<
							0.5<< 0.5<< -0.5<<
							-0.5<<0.5<<-0.5
							<<0.5<<-0.5<<-0.5<<
							0.5<< -0.5<< -0.5<<
							0.5<<-0.5<<0.5<<
							0.5<<0.5<<-0.5<<
							0.5<< 0.5<< 0.5<<
							0.5<<0.5<<-0.5<<
							0.5<<-0.5<<0.5<<
							-0.5<< 0.5<< -0.5<<
							-0.5<<-0.5<<0.5<<
							-0.5<<-0.5<<-0.5<<
							-0.5<< -0.5<< 0.5<<
							-0.5<<0.5<<-0.5<<
							-0.5<<0.5<<0.5<<
							0.5<< 0.5<< -0.5<<
							-0.5<< 0.5<<  0.5<<
							-0.5<<  0.5<<  -0.5<<
							-0.5<<  0.5<<  0.5<<
							0.5<<  0.5<<  -0.5<<
							0.5<< 0.5<<  0.5<<
							-0.5<<  -0.5<< -0.5<<
							-0.5<< -0.5<< 0.5<<
							0.5<< -0.5<< -0.5<<
							0.5<< -0.5<< 0.5<<
							0.5<<  -0.5<< -0.5<<
							-0.5<<  -0.5<< 0.5 ;

	glGenBuffers(1, &vbo);
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, vboData.size() * sizeof(float), vboData.data(), GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind


	// create depth texture used for Shadow Mapping
	// format is RGBA as SGX530 does not support depth_textures
	glGenTextures(1, &depthTex);
	glBindTexture(GL_TEXTURE_2D, depthTex);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, shadowMapSize, shadowMapSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

	scale = 1;
	nvert = vboData.size();
}



void Object::setupPlane(float x, float y , float z, float w, float h) {
	type = PLANE;
	QImage tex;
	// load texture
	tex.load("/opt/arinterface/img/plane.png");
	tex = tex.scaledToWidth(tex.width());
	tex = QGLWidget::convertToGLFormat(tex);

	glGenTextures(1, &cubeTexId);
	glBindTexture(GL_TEXTURE_2D, cubeTexId);

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tex.width(), tex.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits());
	glGenerateMipmap(GL_TEXTURE_2D);


	QVector<float> vboData;// = objFile.getVBO();

	//vertices
	vboData << /*-0.5+x << 0.5+y<< 0.5+z<<
							0.5+x<<-0.5+y<<0.5+z<<
							-0.5+x<<-0.5+y<<0.5+z<<

							0.5+x<< -0.5+y<< 0.5+z<<
							-0.5+x<<0.5+y<<0.5+z<<
							0.5+x<<0.5+y<<0.5+z/<<

							*/


//							-0.5+x<< -0.5+y<< -0.5+z<<
//							0.5+x<<-0.5+y<<-0.5+z<<
//							-0.5+x<<0.5+y<<-0.5+z<<
//							0.5+x<< 0.5+y<< -0.5+z<<
//							-0.5+x<<0.5+y<<-0.5+z<<
//							0.5+x<<-0.5+y<<-0.5+z/*<<


//							0.5+x<< -0.5+y<< -0.5+z<<
//							0.5+x<<-0.5+y<<0.5+z<<
//							0.5+x<<0.5+y<<-0.5+z<<
//							0.5+x<< 0.5+y<< 0.5+z<<
//							0.5+x<<0.5+y<<-0.5+z<<
//							0.5+x<<-0.5+y<<0.5+z/*<<

//							-0.5+x<< 0.5+y<< -0.5+z<<
//							-0.5+x<<-0.5+y<<0.5+z<<
//							-0.5+x<<-0.5+y<<-0.5+z<<
//							-0.5+x<< -0.5+y<< 0.5+z<<
//							-0.5+x<<0.5+y<<-0.5+z<<
//							-0.5+x<<0.5+y<<0.5+z/*<<

//							/*0.5+x<< 0.5+y<< -0.5+z<<
//							-0.5+x<< 0.5+y<<  0.5+z<<
//							-0.5+x<<  0.5+y<<  -0.5+z<<
//							-0.5+x<<  0.5+y<<  0.5+z<<
//							0.5+x<<  0.5+y<<  -0.5+z<<
//							0.5+x<< 0.5+y<<  0.5+z<<
							-w+x<<  y<< -h+z<<
							-w+x<< y<< h+z<<
							w+x<< y<< -h+z<<
							w+x<< y<< h+z<<
							w+x<<  y<< -h+z<<
							-w+x<<  y<< h+z ;


	//texture coordinates
	vboData <<	/*0.0f<<0.0f<< 1.0f<<1.0f<< 1.0f<<0.0f<<
							1.0f<<1.0f<< 0.0f<<0.0f<< 0.0f<<1.0f

							<<*/
//							1.0f<<1.0f<< 1.0f<<0.0f<< 0.0f<<1.0f<<
//							0.0f<<0.0f<< 0.0f<<1.0f<< 1.0f<<0.0f//<<

							/*1.0f<<1.0f<< 1.0f<<0.0f<< 0.0f<<1.0f<<
							0.0f<<0.0f<< 0.0f<<1.0f<< 1.0f<<0.0f<<
							0.0f<<0.0f<< 1.0f<<1.0f<< 1.0f<<0.0f<<
							1.0f<<1.0f<< 0.0f<<0.0f<< 0.0f<<1.0f<<

							0.0f<<1.0f<< 1.0f<<0.0f<< 1.0f<<1.0f<<
							1.0f<<0.0f<< 0.0f<<1.0f<< 0.0f<<0.0f<<*/
							1.0f<<0.0f<< 1.0f<<1.0f<< 0.0f<<0.0f<<
							0.0f<<1.0f<< 0.0f<<0.0f<< 1.0f<<1.0f;


	//normals
	vboData <<	/*0<<0<<-1<< 0<<0<<-1<< 0<<0<<-1<<
							0<<0<<-1<< 0<<0<<-1<< 0<<0<<-1

							<<*/
//							0<<0<<1<< 0<<0<<1<< 0<<0<<1<<
//							0<<0<<1<< 0<<0<<1<< 0<<0<<1//<<
/*
							-1<<0<<0<< -1<<0<<0<< -1<<0<<0<<
							-1<<0<<0<< -1<<0<<0<< -1<<0<<0<<
							1<<0<<0<< 1<<0<<0<< 1<<0<<0<<
							1<<0<<0<< 1<<0<<0<< 1<<0<<0<<

							0<<-1<<0<< 0<<-1<<0<< 0<<-1<<0<<
							0<<-1<<0<< 0<<-1<<0<< 0<<-1<<0<<*/
							0<<1<<0<< 0<<1<<0<< 0<<1<<0<<
							0<<1<<0<< 0<<1<<0<< 0<<1<<0;

	glGenBuffers(1, &vbo);
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, vboData.size() * sizeof(float), vboData.data(), GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind


	// create depth texture used for Shadow Mapping
	// format is RGBA as SGX530 does not support depth_textures
	glGenTextures(1, &depthTex);
	glBindTexture(GL_TEXTURE_2D, depthTex);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, shadowMapSize, shadowMapSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

	scale = 1;
	nvert = vboData.size();
}


void Object::setup(ObjReader objFile) {
	QImage tex;
	QVector<float>& vboData = objFile.getVBO();
	hasNormal = objFile.hasNormal;
//	slices = objFile.getSlices();

	glGenBuffers(1, &vbo);
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, vboData.size() * sizeof(float), vboData.data(), GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind

	// load textures
//	for (int i = 0; i < slices.size(); i++) {
//		tex.load(slices[i].path.c_str());
//		tex = tex.scaledToWidth(tex.width());
//		tex = QGLWidget::convertToGLFormat(tex);

//		glGenTextures(1, &slices[i].texId);
//		glBindTexture(GL_TEXTURE_2D, slices[i].texId);

//		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
//		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

//		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tex.width(), tex.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits());
//		glGenerateMipmap(GL_TEXTURE_2D);
//	}

	// create depth texture used for Shadow Mapping
	// format is RGBA as SGX530 does not support depth_textures
	glGenTextures(1, &depthTex);
	glBindTexture(GL_TEXTURE_2D, depthTex);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, shadowMapSize, shadowMapSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

	scale = 1;
	type = OBJ;
	nvert = objFile.nfaces*3; //all faces are triangles
}

void Object::render(QGLShaderProgram& sp) {

	switch(type){
		case CUBE:
				glBindBuffer(GL_ARRAY_BUFFER, vbo);

				sp.enableAttributeArray("vertex");
				sp.setAttributeArray("vertex", 0, 3, 3*sizeof(float));


				sp.enableAttributeArray("tex");
				sp.setAttributeArray("tex", (float*) (108 * sizeof(float)), 2, 2*sizeof(float));


				sp.enableAttributeArray("norm");
				sp.setAttributeArray("norm", (float*) (180 * sizeof(float)), 3, 3*sizeof(float));

				// bind depth our depth texture
				glActiveTexture(GL_TEXTURE1);
				glBindTexture(GL_TEXTURE_2D, depthTex);

				//bind the object texture
				glActiveTexture(GL_TEXTURE0);
				glBindTexture(GL_TEXTURE_2D, cubeTexId);

				//	qDebug() << "slices size " << slices.size();
				//	for (int i = 0; i < slices.size(); i++) {
				//		glBindTexture(GL_TEXTURE_2D, slices[i].texId);
				//		glDrawArrays(GL_TRIANGLES, slices[i].start, slices[i].count);
				//	}

				glDrawArrays(GL_TRIANGLES, 0, 36);

				glBindTexture(GL_TEXTURE_2D, 0);
				glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind

				glBindBuffer(GL_ARRAY_BUFFER, vbo);

				sp.enableAttributeArray("vertex");
				sp.setAttributeArray("vertex", (float*) (288 * sizeof(float)), 3, 3*sizeof(float));


				sp.enableAttributeArray("tex");
				sp.setAttributeArray("tex", (float*) (108 * sizeof(float)), 2, 2*sizeof(float));


				sp.enableAttributeArray("norm");
				sp.setAttributeArray("norm", (float*) (180 * sizeof(float)), 3, 3*sizeof(float));

				// bind depth our depth texture
				glActiveTexture(GL_TEXTURE1);
				glBindTexture(GL_TEXTURE_2D, depthTex);

				//bind the object texture
				glActiveTexture(GL_TEXTURE0);
				glBindTexture(GL_TEXTURE_2D, cubeTexId);

				//	qDebug() << "slices size " << slices.size();
				//	for (int i = 0; i < slices.size(); i++) {
				//		glBindTexture(GL_TEXTURE_2D, slices[i].texId);
				//		glDrawArrays(GL_TRIANGLES, slices[i].start, slices[i].count);
				//	}

				glDrawArrays(GL_TRIANGLES, 0, 36);

				glBindTexture(GL_TEXTURE_2D, 0);
				glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind
				break;
	case TEXT: //TO-DO: fix this text to draw a line loop for each character
				glBindBuffer(GL_ARRAY_BUFFER, vbo);

				sp.enableAttributeArray("vertex");
				sp.setAttributeArray("vertex", 0, 3, 3*sizeof(float));

				// bind depth our depth texture
//				glActiveTexture(GL_TEXTURE1);
//				glBindTexture(GL_TEXTURE_2D, depthTex);
				//bind the object texture
				glActiveTexture(GL_TEXTURE0);

				glDrawArrays(GL_LINE_LOOP, 0, nvert);

				glBindTexture(GL_TEXTURE_2D, 0);
				glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind

	case PLANE:
			glBindBuffer(GL_ARRAY_BUFFER, vbo);

			sp.enableAttributeArray("vertex");
			sp.setAttributeArray("vertex", 0, 3, 3*sizeof(float));


			sp.enableAttributeArray("tex");
			sp.setAttributeArray("tex", (float*) (18 * sizeof(float)), 2, 2*sizeof(float));


			sp.enableAttributeArray("norm");
			sp.setAttributeArray("norm", (float*) (30 * sizeof(float)), 3, 3*sizeof(float));

			// bind depth our depth texture
			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, depthTex);

			//bind the object texture
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, cubeTexId);

			//	qDebug() << "slices size " << slices.size();
			//	for (int i = 0; i < slices.size(); i++) {
			//		glBindTexture(GL_TEXTURE_2D, slices[i].texId);
			//		glDrawArrays(GL_TRIANGLES, slices[i].start, slices[i].count);
			//	}

			glDrawArrays(GL_TRIANGLES, 0, 6);

			glBindTexture(GL_TEXTURE_2D, 0);
			glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind

			break;
	case OBJ:
			glBindBuffer(GL_ARRAY_BUFFER, vbo);


			if(hasNormal)
			{
				sp.enableAttributeArray("vertex");
				sp.setAttributeArray("vertex", 0, 3, 6 * sizeof(float));

				sp.enableAttributeArray("norm");
				sp.setAttributeArray("norm", (float*) (3 * sizeof(float)), 3, 6 * sizeof(float));
			}
			else
			{
				sp.enableAttributeArray("vertex");
				sp.setAttributeArray("vertex", 0, 3, 3 * sizeof(float));
			}

//			sp.enableAttributeArray("tex");
//			sp.setAttributeArray("tex", (float*) (6 * sizeof(float)), 2, 8 * sizeof(float));

			// bind depth our depth texture
			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, depthTex);

			glActiveTexture(GL_TEXTURE0);

//			for (int i = 0; i < slices.size(); i++) {
//				glBindTexture(GL_TEXTURE_2D, slices[i].texId);
//				glDrawArrays(GL_TRIANGLES, slices[i].start, slices[i].count);
//			}
			glDrawArrays(GL_TRIANGLES, 0, nvert);

			glBindTexture(GL_TEXTURE_2D, 0);
			glBindBuffer(GL_ARRAY_BUFFER, 0); // unbind

	}


}

void Object::cleanup() {
	glDeleteBuffers(1, &vbo);
	glDeleteTextures(1, &depthTex);
	// should delete textures too
}
