/*
 * Augmented Reality For Maemo
 * Based on the code available at https://code.launchpad.net/~rojtberg/arinterface/ by pavel
 *
 * Author: Tassio Knop
 */

#include "View.h"

#ifdef NO_ES
#include <GL/gl.h>
#else
#include <GLES2/gl2.h>
#endif

#include <iostream>
#include <cmath>

#include <QtGui>
#include <QTimer>
#include <QVector3D>

#ifdef Q_WS_MAEMO_5
#include <QtGui/QX11Info>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#endif

using namespace std;

#define FPS 30
//#define RECORDING

View::View(Scene& _s, QWidget* p) :
    QGLWidget(QGLFormat(QGL::DepthBuffer | QGL::AlphaChannel /*| QGL::SampleBuffers*/), p), s(_s), frameCnt(0) {
    setWindowTitle("AR Interface");

    grabGesture(Qt::PanGesture);
    grabGesture(Qt::PinchGesture);
    grabGesture(Qt::SwipeGesture);

    s.ctx = this;

#ifdef NO_ES
    resize(640, 480);
#else
    setWindowState(Qt::WindowFullScreen);
    resize(424, 240);
#endif
    show();
}

void View::initializeGL() {
#ifdef NO_ES
    glewInit();
#endif
    s.setup();
    start();

#ifdef Q_WS_MAEMO_5
    // grab the zoom event on maemo5
    unsigned char grab = 1;

    Atom atom = XInternAtom(QX11Info::display(), "_HILDON_ZOOM_KEY_ATOM", false);

    XChangeProperty (QX11Info::display(),
            winId(),
            atom,
            XA_INTEGER,
            32,
            PropModeReplace,
            &grab,
            1);
#endif
		glClearColor(0.1f, 0.1f, 0.2f, 1.0f);

		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

		m_fAngle = 0;
		m_fScale = 1;
}

void View::start() {
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(updateGL()));
    timer->start(1000 / FPS);
}

void View::paintGL() {
    if (time.elapsed() > 1000) {
				cout << frameCnt << "FPS " << endl;
        time.start();
        frameCnt = 0;
    }
		s.render();

#ifdef RECORDING
		QImage i = grabFrameBuffer();
		imgs.append(i);
#endif
		swapBuffers();
		++frameCnt;

}

void View::keyPressEvent(QKeyEvent *e) {
    if (e->key() == Qt::Key_Left) {
        s.angle += 10;
    } else if (e->key() == Qt::Key_Right) {
        s.angle -= 10;
    } else if (e->key() == Qt::Key_F7) {
        s.activeOb->scale += 0.1;
    } else if (e->key() == Qt::Key_F8) {
        s.activeOb->scale -= 0.1;
    } else if (e->key() == Qt::Key_C) {
        s.bg = s.bg == QColor(0, 0, 0, 0) ? QColor(255, 255, 255, 128) : QColor(0, 0, 0, 0);
    }
		else if (e->key() == Qt::Key_Space)
		{
			s.rotating = !s.rotating;
		}

    s.activeOb->scale = fmax(s.activeOb->scale, 0.0);

    e->accept();
}

bool View::event(QEvent* e) {
    if (e->type() == QEvent::Gesture) {
        cout << "Gesture!" << endl;
    }

    return QWidget::event(e);
}

void View::mousePressEvent(QMouseEvent* e) {
    unsigned char id[4];

    if (e->x() >= 0.9 * width() and e->y() <= 0.1 * height()) {
				 //top right corner
			close();
			return;
    }

    if (e->x() <= 0.1 * width() and e->y() <= 0.1 * height()) {

				s.putCanvas();
				return;
    }


    if (s.annoMode) {
        s.activeCp->newStroke = true;
    } else {
        glReadPixels(e->x(), height() - e->y(), 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, id);

        // TODO this is a hack to filter ids from normal alpha values
        if (id[3] < 10 and id[3] > 0) {
            s.selectObject(id[3] - 1); // convert image id to object id
        }
    }

    QWidget::mousePressEvent(e);
}

void View::mouseMoveEvent(QMouseEvent* e) {
    if (s.annoMode)
        s.activeCp->drawAt(float(e->x()) / width(), float(e->y()) / height());

    QWidget::mouseMoveEvent(e);
}

void View::resizeGL(int w, int h) {
    glViewport(0, 0, w, h);
    s.width = w;
    s.height = h;
}

